<?php declare(strict_types=1);

namespace App\Http\Controllers\Site;

use App\Dto\QuerySettingsDto;
use App\Http\Requests\Site\Repositories\RepositoriesRequest;
use App\Http\Requests\Site\Repositories\StoreUpdateRequest;
use App\Services\Site\RepositoryService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

final class RepositoriesController extends Controller
{
    public function __construct(
        private readonly RepositoryService $repositoryService,
    ) { }

    public function repositories(RepositoriesRequest $request): View
    {
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page:  $request->getDto()->getPage(),
            queryWith: ['user']
        );
        $result = $this->repositoryService->repositories($querySettingsDto);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('site.repositories.repositories', $result->getData());
    }

    public function repository(string $username, string $repositoryName, Request $request): View
    {
        $result = $this->repositoryService->repository($username, $repositoryName, $request->user());
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('site.repositories.repository', $result->getData());
    }

    public function create(string $username, Request $request): View
    {
        $result = $this->repositoryService->create($username, $request->user());
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('site.repositories.create', $result->getData());
    }

    public function edit(string $username, string $repositoryName, Request $request)
    {
        $result = $this->repositoryService->edit($username, $repositoryName, $request->user());
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('site.repositories.edit', $result->getData());
    }

    public function store(string $username, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $result = $this->repositoryService->store($username, $data, $request->user());
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('repository.edit', ['repository_name' => $result->getModel()->name, 'username' => $username])->withSuccess($result->getMessage());
    }

    public function update(string $username, string $repositoryName, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $result = $this->repositoryService->update($username, $repositoryName, $data, $request->user());
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('repository.edit', ['repository_name' => $result->getModel()->name, 'username' => $username])->withSuccess($result->getMessage());
    }

    public function destroy(string $username, string $repositoryName, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->repositoryService->destroy($username, $repositoryName, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('user.repositories', ['username' => $username])->withSuccess($result->getMessage());
    }
}
