<?php declare(strict_types=1);

namespace App\Jobs\Registry\V2;

use App\Dto\Service\Registry\V2\EventService\ImageDataPush;
use App\Exceptions\Jobs\Registry\V2\JobException;
use App\Services\Registry\V2\EventService;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeEncrypted;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;

final class ProcessEventPush implements ShouldQueue, ShouldBeEncrypted
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * The number of times the job may be attempted.
     *
     * @var int
     */
    public $tries = 1000;

    /**
     * Create a new job instance.
     */
    public function __construct(
        public readonly array $target
    ) {
        //
    }

    /**
     * Calculate the number of seconds to wait before retrying the job.
     *
     * @return array<int, int>
     */
    public function backoff(): array
    {
        return [10, 60, 600, 1800, 3600];
    }

    /**
     * Execute the job.
     */
    public function handle(EventService $eventService): void
    {
        [$username, $repositoryName] = \explode('/', $this->target['repository'], 2);

        $size = (int) $this->target['size'] ?? 0;
        $references = $this->target['references'] ?? [];
        foreach ($references as $reference) {
            $size += (int) $reference['size'] ?? 0;
        }

        $data = new ImageDataPush(
            username:       $username,
            repositoryName: $repositoryName,
            tag:            $this->target['tag'],
            digest:         $this->target['digest'],
            size:           $size,
        );
        $result = $eventService->push($data);
        if ($result->isError()) {
            throw new JobException($result->getMessage());
        }
    }
}
