<?php declare(strict_types=1);

namespace App\Services\Registry\V2;

use App\Exceptions\Services\Registry\V2\ArchitectureCommand\BlobException;
use App\Exceptions\Services\Registry\V2\ArchitectureCommand\ManifestException;
use App\Models\Architecture;
use App\Models\OperatingSystem;
use App\ServiceResults\Registry\V2\ArchitectureResult;
use App\ServiceResults\ServiceResultError;
use App\Services\Registry\V2\Api\ApiService;
use App\Services\Service;

final class ArchitectureCommand extends Service
{
    public function __construct(
        private readonly ApiService $apiService,
    ) { }

    public function execute(string $repositoryRegistry, string $digest): ServiceResultError | ArchitectureResult
    {
        try {
            $manifest = $this->apiService->getManifest($repositoryRegistry, $digest);
            $manifest = $manifest->getData();
            if (!isset($manifest['config']) || !isset($manifest['config']['digest'])) {
                throw new ManifestException('Got an empty result');
            }

            $manifestDigest = $manifest['config']['digest'];
            unset($manifest);
            $data = $this->apiService->getBlob($repositoryRegistry, $manifestDigest);
            $data = $data->getData();

            if (!isset($data['architecture']) || !isset($data['os'])) {
                throw new BlobException('Didn\'t receive information about architecture and OS');
            }

            $architecture = $data['architecture'];
            $os = $data['os'];
            unset($data);

            /**
             * @var OperatingSystem $osModel
             */
            $osModel = OperatingSystem::firstOrCreate([
                'name' => $os,
            ]);

            /**
             * @var Architecture $architectureModel
             */
            $architectureModel = $osModel->architectures()->firstOrCreate([
                'name' => $architecture,
            ]);
        } catch (\Throwable $e) {
            report($e);
            return $this->errService($e->getMessage());
        }

        return new ArchitectureResult($architectureModel);
    }
}
