<?php declare(strict_types=1);

namespace App\Services\Site;

use App\Dto\QuerySettingsDto;
use App\Dto\Service\Site\AccessTokens\StoreUpdate;
use App\Enums\AccessTokenPermission;
use App\Models\User;
use App\Repositories\AccessTokenRepository;
use App\ServiceResults\ServiceResultArray;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\ServiceResultSuccess;
use App\ServiceResults\Site\AccessTokenService\StoreTokenResult;
use App\ServiceResults\StoreUpdateResult;
use App\Services\Service;

final class AccessTokenService extends Service
{
    public function __construct(
        private readonly AccessTokenRepository $accessTokenRepository,
    ) { }

    public function index(QuerySettingsDto $querySettingsDto, User $user): ServiceResultError | ServiceResultArray
    {
        $tokens = $this->accessTokenRepository->getUserTokens(
            $user,
            $querySettingsDto->getQueryWith()
        )->pagination(
            $querySettingsDto->getLimit(),
            $querySettingsDto->getPage()
        );

        return $this->result([
            'tokens' => $tokens
        ]);
    }

    public function create(User $user): ServiceResultError | ServiceResultArray
    {
        return $this->result([
            'user'           => $user,
            'serviceAddress' => config('registry.service_http'),
            'permissions'    => AccessTokenPermission::toCollection()->pluck('title', 'value')->toArray(),
        ]);
    }

    public function edit(int $id, User $user): ServiceResultError | ServiceResultArray
    {
        $token = $this->accessTokenRepository->getTokenById($user, $id);
        if (\is_null($token)) {
            return $this->errNotFound(__('Not Found'));
        }

        return $this->result([
            'token'       => $token,
            'user'        => $user,
            'permissions' => AccessTokenPermission::toCollection()->pluck('title', 'value')->toArray(),
        ]);
    }

    public function store(StoreUpdate $data, User $user): ServiceResultError | StoreTokenResult
    {
        try {
            $token = $user->createToken($data->getName(), [$data->getPermission()->value])->plainTextToken;
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return new StoreTokenResult(
            name:       $data->getName(),
            permission: $data->getPermission(),
            token:      $token,
        );
    }

    public function update(int $id, StoreUpdate $data, User $user): ServiceResultError | StoreUpdateResult
    {
        $token = $this->accessTokenRepository->getTokenById($user, $id);
        if (\is_null($token)) {
            return $this->errNotFound(__('Not Found'));
        }

        try {
            $token->update([
                'name'      => $data->getName(),
                'abilities' => [$data->getPermission()->value],
            ]);
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->resultStoreUpdateModel($token, __('site.Token updated successfully'));
    }

    public function destroy(int $id, User $user): ServiceResultError|ServiceResultSuccess
    {
        $token = $this->accessTokenRepository->getTokenById($user, $id);
        if (\is_null($token)) {
            return $this->errNotFound(__('Not Found'));
        }

        try {
            $token->delete();
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->ok(__('site.The token has been deleted'));
    }
}
